/*	
 *	frampsid.c
 *
 *	P/ECE PSID Driver
 *
 *	CLiP - Common Library for P/ECE
 *	Copyright (C) 2001-2003 Naoyuki Sawa
 *
 *	* Sat Dec 28 21:38:00 JST 2003 Naoyuki Sawa
 *	- 쐬JnB
 */
#include "clip.h"

/****************************************************************************
 *	M6502G~[VpO֐
 ****************************************************************************/

/* d400`d41f = SID register
 * d420`d7ff = SID register images
 */

unsigned char
psid_m6502_read(M6502* m6502, unsigned short addr)
{
	PSIDDRIVER* psid = (PSIDDRIVER*)m6502;
	unsigned char* mem = psid->mem;

	/* SID */
	if(addr >= 0xd400 && addr <= 0xd7ff) {
		return sid_read(&psid->sid, addr & 0x1f);
	}

//#ifdef SIDPLAY_DEBUG
//	do {
//		if(addr < 0xd000) break;
//		if(addr > 0xdfff) break;
//		TRACE("READ : %04x\n", addr);
//	} while(0);
//#endif /*SIDPLAYDEBUG*/

	/* Main Memory */
	return mem[addr];
}

void
psid_m6502_write(M6502* m6502, unsigned short addr, unsigned char data)
{
	PSIDDRIVER* psid = (PSIDDRIVER*)m6502;
	unsigned char* mem = psid->mem;
	int regno;

	/* SID */
	if(addr >= 0xd400 && addr <= 0xd7ff) {
		regno = addr & 0x1f;
		if(regno <= 0x1c) {
			/* Real SID Chip */
			sid_write(&psid->sid, regno, data);
		} else {
			/* PlaySID Extended SID */
			psid_xsid_write(psid, addr, data);
		}
		return;
	}

//#ifdef SIDPLAY_DEBUG
//	do {
//		if(addr < 0xd000) break;
//		if(addr > 0xdfff) break;
//		TRACE("WRITE: %04x %02x\n", addr, data);
//	} while(0);
//#endif /*SIDPLAYDEBUG*/

	/* Main Memory */
	mem[addr] = data;
}

/****************************************************************************
 *	PSIDhCo֐
 ****************************************************************************/

int
psid_mix(PSIDDRIVER* psid, short* wbuff)
{
	M6502* m6502 = &psid->m6502;
	SID* sid = &psid->sid;
	int i_sample;
	int value;
	int step;

	/* tĂяoZbgAbvB */
	psid_m6502_setup(psid, 0, psid->play_addr);
	for(i_sample = 0; i_sample < SIDBUFLEN; i_sample++) {
		/* 1TvAo߃NbN擾B */
		value  = sid_mix1(sid, &step);		/* Real SID    : t17.5bit */
		value += psid_xsid_process(psid, step);	/* Extended SID: t17.5bit */

		/* XP[OENbsOĊi[B */
		value >>= 1; /*  */
		if(value < -0x7fff) value = -0x7fff;
		if(value >  0x7fff) value =  0x7fff;
		*wbuff++ = value;

		/* o߃NbN̕ACPUs܂B */
		m6502_run(m6502, m6502->cycle + step);
	}
	/* play_addr̂荞ݏȂ̂ŁAm6502_int()͕svłB */

	return 0;
}

